#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/base/base.rb'
# module ImageParadise; class Foobar < Base
# =========================================================================== #
module ImageParadise

class Base # === ImageParadise::Base

  require 'image_paradise/requires/common_base_requires.rb'
  require 'image_paradise/constants/constants.rb'
  require 'image_paradise/toplevel_methods/esystem.rb'
  require 'image_paradise/toplevel_methods/file_related_code.rb'
  require 'image_paradise/toplevel_methods/misc.rb'
  require 'image_paradise/toplevel_methods/roebe.rb'

  include Colours

  N = "\n" # Newline.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    Signal.trap('SIGINT') {
      e 'Exiting now, as requested.'
      exit
    }
  end

  # ========================================================================= #
  # === write_what_into
  # ========================================================================= #
  def write_what_into(what, into)
    ::SaveFile.write_what_into(what, into)
  end

  # ========================================================================= #
  # === esystem
  # ========================================================================= #
  def esystem(i)
    ::ImageParadise.esystem(i)
  end

  # ========================================================================= #
  # === eparse
  # ========================================================================= #
  def eparse(i)
    Colours.eparse(i)
  end

  # ========================================================================= #
  # === mkdir
  # ========================================================================= #
  def mkdir(i)
    FileUtils.mkdir_p(i) unless File.directory? i
  end; alias mkdir_p mkdir # === mkdir_p

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === today
  # ========================================================================= #
  def today
    "#{Time.now.strftime('%d.%m.%Y')}_#{Time.now.strftime('%H:%M:%S.%L')}"
  end

  # ========================================================================= #
  # === increment_this_number_by_one
  # ========================================================================= #
  def increment_this_number_by_one(i)
    ::ImageParadise.increment_this_number_by_one(i)
  end

  # ========================================================================= #
  # === output_file_exists?
  # ========================================================================= #
  def output_file_exists?(
      i = name_of_the_output_file?
    )
    File.exist? i.to_s
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    @commandline_arguments.first
  end; alias first? first_argument? # === first?

  # ========================================================================= #
  # === first_argument_without_hyphen?
  # ========================================================================= #
  def first_argument_without_hyphen?
    _ = @commandline_arguments.select {|entry| !entry.start_with? '--' }
    if _
      _.first
    else
      nil
    end
  end

  # ========================================================================= #
  # === commandline_arguments_starting_with_hyphens?
  # ========================================================================= #
  def commandline_arguments_starting_with_hyphens?
    return @commandline_arguments.select {|entry| entry.start_with? '--' }
  end; alias commandline_arguments_via_leading_hyphens commandline_arguments_starting_with_hyphens? # === commandline_arguments_via_leading_hyphens

  # ========================================================================= #
  # === commandline_arguments_as_stripped_string
  #
  # This variant will return the commandline arguments as a String.
  #
  # Note that any --hyphened argument will be ignored.
  # ========================================================================= #
  def commandline_arguments_as_stripped_string
    @commandline_arguments.reject {|entry|
      entry.start_with?('--')
    }.join(' ').strip
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn(
      i = @namespace
    )
    if i.is_a? String
      i = { namespace: i }
    end
    Opn.opn(i)
  end

  # ========================================================================= #
  # === is_on_roebe?
  # ========================================================================= #
  def is_on_roebe?
    ::ImageParadise.is_on_roebe?
  end

  # ========================================================================= #
  # === return_pwd
  # ========================================================================= #
  def return_pwd
    rds("#{Dir.pwd}/")
  end

  # ========================================================================= #
  # === chdir                                                        (cd tag)
  # ========================================================================= #
  def chdir(i)
    Dir.chdir(i)
  end; alias cd chdir # === cd

  # ========================================================================= #
  # === rds
  #
  # rds is an abbreviation for "remove double slashes", e. g. "//" to
  # to become "/".
  # ========================================================================= #
  def rds(i)
    if i.is_a? Array
      i.map {|entry| rds(entry) }
    else
      i.squeeze('/')
    end
  end

  # ========================================================================= #
  # === return_all_images_from
  # ========================================================================= #
  def return_all_images_from(this_directory)
    target = rds(this_directory+'/**/**')
    Dir[target].select {|entry|
      is_an_image_file?(entry)
    }
  end

  # ========================================================================= #
  # === is_an_image_file?
  # ========================================================================= #
  def is_an_image_file?(i)
    ::ImageParadise.is_an_image_file?(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = ImageParadise::Base.new
  puts 'Is .png an image file: '+
       _.is_an_image_file?('foobar.png').to_s  
end