#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/gui/gtk3/image_editor/connect_skeleton.rb'
# =========================================================================== #
require 'gtk_paradise/require_gtk3'

module ImageParadise

module GUI

module Gtk

class ImageEditor < ::Gtk::Box # === ImageParadise::GUI::Gtk::ImageEditor 

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    most_outer_vbox = gtk_vbox

    vbox1 = gtk_vbox
    vbox1.minimal(@button_flip_the_image,             2)
    vbox1.minimal(@button_greyscale_this_image,       2)
    vbox1.minimal(@button_make_the_image_transparent, 2)
    vbox1.minimal(@button_negate,                     2)
    vbox1.minimal(@button_create_drop_shadow_effect,  2)
    @button_box1.minimal(vbox1, 2)

    vbox2 = gtk_vbox
    vbox2.minimal(@button_circle_mask, 2)
    vbox2.minimal(@button_torn_paper_effect, 2)
    vbox2.minimal(@button_shadow_effect, 2)
    vbox2.minimal(@button_chop_away_n_pixel, 2)
    vbox2.minimal(@button_blur, 2)

    @button_box1.minimal(vbox2, 2)
    @button_box1.minimal(return_vbox_containing_the_round_the_corner_button_and_other_buttons,    2)
    @button_box1.minimal(return_vbox_containing_the_colour_filled_image_button_and_other_buttons, 2)
    @button_box1.minimal(return_widget_with_more_buttons_starting_with_the_sharpen_button,        2)
    @button_box1.minimal(return_the_widget_containing_the_button_create_border,                   2)

    mini_vbox = gtk_vbox
    mini_vbox.minimal(return_widget_containing_the_write_text_functionality, 2)
    # ======================================================================= #
    # Next add this tiny vbox widget:
    # ======================================================================= #
    @button_box1.minimal(mini_vbox, 0)
    @box_for_the_main_image.maximal(@work_on_this_image)
    @box_for_the_main_image.css_class('pad10px')
    # ======================================================================= #
    # Create the scrolled-window for our main image next.
    # ======================================================================= #
    create_the_scrolled_window_and_associated_event_box_for_the_box_for_the_main_image
    most_outer_vbox.minimal(@header_bar, 2)

    # ======================================================================= #
    # Next we will create the vbox that holds the widget containing
    # information about the image.
    # ======================================================================= #
    vbox_containing_info_and_tool_buttons = gtk_vbox
    vbox_containing_info_and_tool_buttons.minimal(
      return_widget_containing_information_about_an_image, 1)
    vbox_containing_info_and_tool_buttons.minimal(
      return_widget_containing_the_undo_button,            1)
    vbox_containing_info_and_tool_buttons.minimal(
      @spin_button_for_the_first_argument,                 1)
    vbox_containing_info_and_tool_buttons.minimal(
      @spin_button_for_the_second_argument,                1)
    vbox_containing_info_and_tool_buttons.minimal(
      @button_colour_button,                               1)
    pane_top_to_bottom1 = gtk_top_to_bottom_pane(
      vbox_containing_info_and_tool_buttons,
      @scrolled_window_for_the_box_for_the_main_image
    )
    pane_top_to_bottom1.position = (width? - 1420)

    hbox_resize_the_image_and_entry = gtk_hbox
    hbox_resize_the_image_and_entry.minimal(@button_resize_the_image, 2)
    @entry_rotate_n_percent = entry('50')
    @spin_button_percentage_value = gtk_spin_button(1, 250, 1)
    @spin_button_percentage_value.to_the_middle
    @spin_button_percentage_value.start_value = 50
    @spin_button_percentage_value.on_changed {
      # On changed, sync it with the entry below.
      _ = @spin_button_percentage_value.text?.to_s
      @entry_rotate_n_percent.set_text(_)
    }
    @entry_rotate_n_percent.hint = 'This says how '\
      'much to shrink or increase the image-size, '\
      'in <b>percent</b>.'
    hbox_resize_the_image_and_entry.minimal(@entry_rotate_n_percent, 2)
    @entry_rotate_n_percent.center
    @entry_rotate_n_percent.width_height(48, 18)
    hbox_resize_the_image_and_entry.minimal(text('%'), 2)
    hbox_resize_the_image_and_entry.minimal(@spin_button_percentage_value, 2)

    @button_box2.minimal(return_rotate_the_image_widget, 2)
    @button_box2.minimal(return_the_widget_containing_the_crop_button_and_the_associated_entries, 2)
    @button_box2.minimal(return_draw_rectangle_widget, 2)
    @button_box2.minimal(return_the_widget_containing_the_draw_line_functionality, 2)
    @button_box2.minimal(hbox_resize_the_image_and_entry, 2)

    vbox = gtk_vbox
    vbox.minimal(@button_box1, 2) # This contains e. g. the "flip horizontally" button.
    vbox.minimal(@button_box2, 2)
    _ = gtk_vbox
    _.minimal(pane_top_to_bottom1, 2)
    _.minimal(return_hbox_containing_the_entry_work_on_the_following_image_and_the_open_in_browser_event_box, 2)
    left_to_right_pane = gtk_left_to_right_pane(_, vbox)
    # ======================================================================= #
    # Next add the left-to-right pane that will contain the image that we
    # may modify.
    # ======================================================================= #
    most_outer_vbox.minimal(left_to_right_pane)
    # ======================================================================= #
    # Embed this all into a scrolling-window next:
    # ======================================================================= #
    scrolled_window = gtk_scrolled_window(most_outer_vbox) { :only_top_to_bottom }
    scrolled_window.width_height(480, 480)
    maximal(scrolled_window)
  end

end; end; end; end