#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::Identify
#
# The purpose of this class is to simply gather all information that
# we can obtain by ImageMagick's identify command.
#
# Currently, we use the plain "identify" binary. We could also use
# RMagick, but for now we shall not do so.
# =========================================================================== #
# require 'image_paradise/identify.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class Identify < ::ImageParadise::Base

  # ========================================================================= #
  # === CMD_TO_RUN
  # ========================================================================= #
  CMD_TO_RUN = 'identify -verbose' # Could also be "identify".
  
  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_image_file = 'test.png',
      run_already     = true
    )
    reset
    set_use_this_image_file(this_image_file)
    run if run_already
  end

  # ========================================================================= #
  # === set_use_this_image_file
  # ========================================================================= #
  def set_use_this_image_file(i)
    if i.is_a? Array
      i = i.first
    end
    @use_this_image_file = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @result
    # ======================================================================= #
    @result = nil
    # ======================================================================= #
    # === @colorspace
    # ======================================================================= #
    @colorspace = nil
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e @result
  end

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    e
    report_internal_attributes {{ padding: 2 }}
    e
  end

  # ========================================================================= #
  # === report_internal_attributes
  # ========================================================================= #
  def report_internal_attributes
    padding_to_use = ''.dup
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        # =================================================================== #
        # === :padding
        # =================================================================== #
        if yielded.has_key? :padding
          padding_to_use << ' ' * yielded[:padding]
        end
      end
    end
    n_just = 18
    e steelblue(padding_to_use+'Mime type: '.ljust(n_just))+
      mediumseagreen(@mime_type)
    e steelblue(padding_to_use+'Width: '.ljust(n_just))+
      mediumseagreen(@width)
    e steelblue(padding_to_use+'Height: '.ljust(n_just))+
      mediumseagreen(@height)
    e steelblue(padding_to_use+'Background color: '.ljust(n_just))+
      mediumseagreen(@background_color)
    e steelblue(padding_to_use+'Resolution: '.ljust(n_just))+
      mediumseagreen(@resolution)
    e steelblue(padding_to_use+'Quality: '.ljust(n_just))+
      mediumseagreen(@quality)
    e steelblue(padding_to_use+'Colorspace: '.ljust(n_just))+
      mediumseagreen(@colorspace)
  end

  # ========================================================================= #
  # === analyze_result
  # ========================================================================= #
  def analyze_result
    _ = @use_this_image_file
    if _ and File.exist?(_)
      @result = `#{CMD_TO_RUN} #{_}`
    else
      e 'No file could be found. Please provide a valid '\
        'argument to this class.'
    end
    if @result
      # Next we will scan for useful information.
      @mime_type        = @result.scan(/Mime type:(.+)$/).flatten.first.strip
      @width, @height   = @result.scan(/Geometry:(.+)$/).flatten.first.split('x').map(&:strip)
      @resolution       = @result.scan(/Resolution:(.+)$/).flatten.first.strip
      @background_color = @result.scan(/Background color:(.+)$/).flatten.first.strip
      @quality          = @result.scan(/Quality:(.+)$/).flatten.first.strip
      @colorspace       = @result.scan(/Colorspace:(.+)$/).flatten.first.strip
      report
    end
  end
 
  # ========================================================================= #
  # === report_everything
  # ========================================================================= #
  def report_everything
    report_internal_attributes
    report_result
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= # 
  def run
    analyze_result
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::Identify.new(ARGV)
end # rubyidentify