#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::ImageParadise
#
# This project can be used to do fancy image manipulations from the
# commandline, including the de-novo creation of new images.
#
# Usage example:
#
#   ImageParadise::ImageParadise.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise'
# =========================================================================== #
require 'image_paradise/requires/common_base_requires.rb'
require 'image_paradise/toplevel_methods/esystem.rb'
require 'image_paradise/toplevel_methods/crop.rb'
require 'image_paradise/toplevel_methods/png_to_svg.rb'
require 'image_paradise/identify.rb'
require 'image_paradise/graphs.rb'
require 'image_paradise/confree_generator/confree_generator.rb'
require 'image_paradise/optimizer.rb'

module ImageParadise

class ImageParadise # === ImageParadise::ImageParadise

  require 'image_paradise/constants/constants.rb'
  require 'image_paradise/toplevel_methods/create_animated_gif.rb'

  # ========================================================================= #
  # === initialize
  #
  # First argument is the filename that we wish to use.
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_filename(i)
    if run_already.to_s.include? 'dont'
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_filename
  #
  # This will be the input filename.
  # ========================================================================= #
  def set_filename(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @filename = i
  end

  # ========================================================================= #
  # === filename?
  # ========================================================================= #
  def filename?
    @filename
  end; alias input? filename? # === input?

  # ========================================================================= #
  # === ImageParadise.font_tiled
  # ========================================================================= #
  def self.font_tiled(i = DEFAULT_TEXT)
    i = DEFAULT_TEXT if i.nil?
    width_and_height = '500x250'
    _ = CONVERT+" -size "+width_and_height+" xc:lightblue -font "+
        DEFAULT_FONT+" -pointsize 68 
        -tile pattern:checkerboard -annotate +28+68 '"+i+"' 
        font_tile.jpg"
    esys _
  end

  # ========================================================================= #
  # === ImageParadise.esys
  # ========================================================================= #
  def self.esys(i)
    i.delete!(N)
    esystem(i)
  end

  # ========================================================================= #
  # === esys
  # ========================================================================= #
  def esys(i)
    ImageParadise.esys(i)
  end

  # ========================================================================= #
  # === colourize
  #
  # Input should be R,G,B values.
  # ========================================================================= #
  def colourize(rgb_values = '2,15,75')
    case rgb_values
    when :randomize, :random
      rgb_values = (rand(255)+1).to_s+','+(rand(255)+1).to_s+','+(rand(255)+1).to_s
    end
    _ = CONVERT+' -colorize '+rgb_values+' '+input?+' '+output?
    ImageParadise.esys _
  end

  # ========================================================================= #
  # === output
  # ========================================================================= #
  def output?
    'output.png'
  end

  # ========================================================================= #
  # === ImageParadise.resize
  #
  # This is equivalent to:
  #   convert terminal.gif  -resize 64x64  resize_terminal.gif
  # ========================================================================= #
  def self.resize(resize_how = '-resize 64x64')
    _ = CONVERT+' '+resize_how+' '+input_output?
  end

  # ========================================================================= #
  # === input_output?
  # ========================================================================= #
  def input_output?
    input?+' '+output?
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # (run tag)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  # _ = ImageParadise.new(ARGV, :dont_run_yet)
  # _.run
  ImageParadise::ImageParadise.crop('rose.jpg')
end # rimage
    # require 'image_paradise'; ImageParadise.new('foo.jpg').colourize :random