#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::ImageToPdf
#
# This class will simply convert an image-file into a .pdf "file".
#
# Usage example:
#
#   ImageParadise::ImageToPdf.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise/image_to_pdf/image_to_pdf.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class ImageToPdf < Base # === ImageParadise::ImageToPdf

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    @commandline_arguments.first
  end; alias first? first_argument? # === first?
       alias input? first_argument? # === input?

  # ========================================================================= #
  # === output?
  # ========================================================================= #
  def output?
    input?.gsub(/#{extname?}/,'')+'.pdf'
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === extname?
  # ========================================================================= #
  def extname?
    File.extname(input?)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = "convert #{input?} #{output?}"
    esystem _
  end

  # ========================================================================= #
  # === ImageParadise::ImageToPdf[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::ImageToPdf.new(ARGV)
end # image_to_pdf