#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::SimpleLabel
#
# This class will create a simple label.
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class SimpleLabel < Base # === ImageParadise::SimpleLabel

  begin
    require 'open'
  rescue LoadError; end

  # ========================================================================= #
  # === DO_OPEN_IN_BROWSER
  # ========================================================================= #
  DO_OPEN_IN_BROWSER = true

  # ========================================================================= #
  # === DEFAULT_FONT_TO_USE
  # ========================================================================= #
  DEFAULT_FONT_TO_USE = 'Helvetica'

  # ========================================================================= #
  # === DEFAULT_IMAGE_FORMAT
  # ========================================================================= #
  DEFAULT_IMAGE_FORMAT = 'PNG'

  # ========================================================================= #
  # === REGEX_FOR_SIZE
  # ========================================================================= #
  REGEX_FOR_SIZE = /(\d+x\d+)/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_label(i)
    run if run_already
  end

  # ========================================================================= #
  # === set_label
  # ========================================================================= #
  def set_label(i)
    if i.is_a? Array
      i = i.join(' ').strip
    end
    # ======================================================================= #
    # Check whether the label includes a Dimension. This may be a conflict
    # if the user REALLY wants to use some integer value, as part of the
    # given label - but aside from this, I think in most other cases,
    # this is a good default.
    # ======================================================================= #
    if i =~ REGEX_FOR_SIZE
      size_dimension = $1.to_s.dup
      set_size(size_dimension)
      i.sub!(/#{size_dimension}/,'')
    end
    @label = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @_           = ''.dup
    @label       = ''.dup
    @background  = 'lightblue'
    @fill        = 'blue'
    @font        = DEFAULT_FONT_TO_USE
    @pointsize   = '96'
    @output_file = 'label.'+DEFAULT_IMAGE_FORMAT.downcase
    set_size       '250x75'
  end

  # ========================================================================= #
  # === set_size
  # ========================================================================= #
  def set_size(i = '250x75')
    @size = i.to_s
  end

  # ========================================================================= #
  # === build_up_main_command
  # ========================================================================= #
  def build_up_main_command
    @_ = 'convert'.dup
    add_default_background
    add_default_fill
    add_default_font
    add_default_gravity
    add_size
    add_label
    # add_default_pointsize # Do not add this option if you also added the default size.
    add_output_file
  end

  # ========================================================================= #
  # === add_size
  # ========================================================================= #
  def add_size
    @_ << ' -size '+@size
  end

  # ========================================================================= #
  # === add_default_background
  # ========================================================================= #
  def add_default_background
    @_ << ' -background '+@background
  end

  # ========================================================================= #
  # === add_default_font
  # ========================================================================= #
  def add_default_font
    @_ << ' -font '+@font
  end

  # ========================================================================= #
  # === add_label
  # ========================================================================= #
  def add_label
    _ = @label
    if _.include? ' '
      _ = '"'+_+'"'
    end
    @_ << ' label:'+_
  end

  # ========================================================================= #
  # === add_default_gravity
  # ========================================================================= #
  def add_default_gravity
    @_ << ' -gravity center'
  end

  # ========================================================================= #
  # === output_then_run_the_main_command
  # ========================================================================= #
  def output_then_run_the_main_command
    esystem @_
  end

  # ========================================================================= #
  # === add_output_file
  # ========================================================================= #
  def add_output_file
    @_ << ' '+@output_file
  end

  # ========================================================================= #
  # === output_file?
  # ========================================================================= #
  def output_file?
    @output_file
  end

  # ========================================================================= #
  # === add_default_pointsize
  # ========================================================================= #
  def add_default_pointsize
    @_ << ' -pointsize '+@pointsize
  end

  # ========================================================================= #
  # === add_default_fill
  # ========================================================================= #
  def add_default_fill
    @_ << ' -fill '+@fill
  end

  # ========================================================================= #
  # === consider_opening_in_the_browser
  # ========================================================================= #
  def consider_opening_in_the_browser
    if DO_OPEN_IN_BROWSER
      Open.in_browser(output_file?)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    build_up_main_command
    output_then_run_the_main_command
    consider_opening_in_the_browser
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::SimpleLabel.new(ARGV)
end # simple_label "Tomorrow is another day."
    # simple_label "Tomorrow is another day. 500x250"