#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::Interactive
#
# Usage example:
#   ImageParadise::Interactive.new
# =========================================================================== #
# require 'image_paradise/shell/interactive.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'
require 'image_paradise/toplevel_methods/write_this_text.rb'

module ImageParadise

class Interactive < ::ImageParadise::Base # === ImageParadise::Interactive

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    register_sigint
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = [i] unless i.is_a? Array
    @input = i # Must be an Array.
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    try_to_enter_this_directory('/Depot/j')
    loop {
      obtain_user_input
      menu(@user_input)
    }
  end

  # ========================================================================= #
  # === try_to_enter_this_directory
  # ========================================================================= #
  def try_to_enter_this_directory(i = '/Depot/j')
    if File.directory?(i)
      chdir(i)
    end
  end

  # ========================================================================= #
  # === obtain_user_input
  # ========================================================================= #
  def obtain_user_input
    @user_input = $stdin.gets.chomp
  end

  # ========================================================================= #
  # === display_prompt
  # ========================================================================= #
  def display_prompt
    print '> '
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @input
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      display_prompt
      case i # case tag
      # ===================================================================== #
      # === pwd
      # ===================================================================== #
      when /pwd/
        e sdir(Dir.pwd)
      # ===================================================================== #
      # === bachelor_master
      # ===================================================================== #
      when '1',/bachelor_?master/
        generate_bachelor_and_master_images
      when 'q','exit'
        exit
      end
    end
  end

  # ========================================================================= #
  # === generate_bachelor_and_master_images                        (main tag)
  # ========================================================================= #
  def generate_bachelor_and_master_images
    height_value    =  28
    width_value     =  94
    pointsize_value =  18
    # ======================================================================= #
    # Designate another store-location for our two images there.
    # ======================================================================= #
    ImageParadise.store_image_files_in_this_directory = 
      '/home/x/DATA/IMG/STUDIUM/'
    ImageParadise.write_this_text(
      'Bachelor',
        {colour: 'red', height: height_value, width: width_value, upcase_filename: true, pointsize: pointsize_value}
    )
    ImageParadise.write_this_text(
      'Master',
        {colour: 'darkblue', height: height_value, width: width_value, upcase_filename: true, pointsize: pointsize_value}
    )
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === ImageParadise.interactive_shell
# =========================================================================== #
def self.interactive_shell
  ImageParadise::Interactive.new
end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::Interactive.new(ARGV)
end # interactive.rb