#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Shared coded for the SVG component of the ImageParadise project
# be stored here.
# =========================================================================== #
# require 'image_paradise/svg/svg.rb'
# =========================================================================== #
module ImageParadise

module Svg # === ImageParadise::Svg

  require 'image_paradise/project/project.rb'
  require 'image_paradise/svg/circle.rb'
  require 'image_paradise/svg/feature.rb'
  require 'image_paradise/svg/rectangle.rb'

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === @_
  #
  # This is a variable that will keep the big String.
  # ========================================================================= #
  @_ = ''.dup

  # ========================================================================= #
  # === Svg.reset
  # ========================================================================= #
  def self.reset
    @_ = ''
  end

  # ========================================================================= #
  # === Svg.strip
  # ========================================================================= #
  def self.strip(i)
    return i.squeeze(' ')
  end

  # ========================================================================= #
  # === Svg.string?
  # ========================================================================= #
  def self.string?
    @_
  end; self.instance_eval { alias [] string? } # === Svg[]

  # ========================================================================= #
  # === Svg.add
  #
  # Simply append onto @_.
  # ========================================================================= #
  def self.add(i = '')
    i = i.to_s.dup
    @_ << i
  end

  # ========================================================================= #
  # === Svg.close_svg
  #
  # Simply close the <svg> tag here
  # ========================================================================= #
  def self.close_svg
    return '</svg>'+N
  end

  # ========================================================================= #
  # === Svg.chart_directory
  #
  # Give us the path towards the svg_-charts subdirectory.
  # ========================================================================= #
  def self.chart_directory
    ::ImageParadise.project_base_dir?+'charts/'
  end

  # ========================================================================= #
  # === Svg.load_chart_module
  #
  # This method will load the individual .rb files from the chart/
  # subdirectory. Currently this is not in use though - perhaps one
  # day when I get to use the chart-svg subsection.
  # ========================================================================= #
  def self.load_chart_module
    these_files = Svg.chart_directory+'*.rb'
    Dir[these_files].each {|file|
      file = 'image_paradise/svg/charts/'+File.basename(file)
      require file # And require it.
    }
  end; load_chart_module # And call it at once.

  # ========================================================================= #
  # === Svg.add
  # ========================================================================= #
  def self.add_linebreak
    add "<br>#{N}"
  end

end; end