#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/toplevel_methods/add_black_border_to_this_image.rb'
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/e.rb'
  require 'image_paradise/toplevel_methods/esystem.rb'

  # ========================================================================= #
  # === ImageParadise.add_black_border_to_this_image
  #
  # This method will add a (black) border around the given input-image.
  #
  # Since as of February 2021 this does no longer have to be confined
  # solely to black as colour; other colours can be used as well. See
  # the third argument to this method.
  #
  # The -bordercolor commandline can be used to specify the colour
  # of the border at hand.
  #
  # The -border 5x10 variant means to have a 5px border left-right,
  # and a 10px border top-bottom, each. Note that a single value
  # of 5 here means exactly the same as 5x5. USe the variant you
  # prefer.
  #
  # Invocation example:
  #
  #   ImageParadise.add_black_border_to_this_image('foobar.png',3)
  #
  # ========================================================================= #
  def self.add_black_border_to_this_image(
      file_that_is_to_be_modified = 'foobar.png',
      n_pixels                    = '1',
      use_this_colour             = 'black', # Provide the colour here.
      new_filename                = nil
    )
    n_pixels = n_pixels.to_s
    if file_that_is_to_be_modified.is_a? Array
      file_that_is_to_be_modified = file_that_is_to_be_modified.first
    end
    if File.exist? file_that_is_to_be_modified
      extname = File.extname(file_that_is_to_be_modified) # Also has a leading '.' character.
      unless new_filename
        new_filename = file_that_is_to_be_modified.sub(
          /#{File.extname(file_that_is_to_be_modified)}/,''
        )
        new_filename << '_'+use_this_colour.to_s+'_border_'+n_pixels+'px'
        new_filename << extname
      end
      _ = "convert #{file_that_is_to_be_modified} "\
          "-density 300 "\
          "-shave #{n_pixels}x#{n_pixels} "\
          "-bordercolor '#{use_this_colour}' "+ # Specify the colour of the border.
          "-border #{n_pixels} "\
          "#{new_filename}"
      esystem _
      if File.exist? new_filename
        e "The file `#{new_filename}` has been created."
      end
      return new_filename # Return it here as well.
    else
      e 'No file exists at `'+file_that_is_to_be_modified+'`.'
    end
  end; self.instance_eval { alias add_border_to_this_image add_black_border_to_this_image } # === ImageParadise.add_border_to_this_image('foobar.png', 3, 'red')

  # ========================================================================= #
  # === ImageParadise.add_green_border_to_this_image
  #
  # Usage example:
  #
  #   ImageParadise.add_green_border_to_this_image('TANAKA.png','12')
  #
  # ========================================================================= #
  def self.add_green_border_to_this_image(
      file_that_is_to_be_modified = 'foobar.png',
      n_pixels                    = '1',
      new_filename                = nil
    )
    ::ImageParadise.add_border_to_this_image(
      file_that_is_to_be_modified,
      n_pixels,
      'green',
      new_filename
    )
  end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.add_black_border_to_this_image(ARGV)
end # bordblack1 STUDIES.png