#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/toplevel_methods/extract_text_from_this_image.rb'
# =========================================================================== #
require 'image_paradise/toplevel_methods/esystem.rb'

module ImageParadise

  # ========================================================================= #
  # === ImageParadise.extract_text_from_this_image
  #
  # This method will use tesseract to extract/return text from the
  # given image at hand.
  #
  # Invocation example:
  #
  #   ImageParadise.extract_text_from_this_image('foobar.png')
  #   ImageParadise.extract_text_from_this_image('greyscaled_foo.jpg')
  #   ImageParadise.extract_text_from_this_image 'greyscaled_image.png'
  #
  # ========================================================================= #
  def self.extract_text_from_this_image(
      i,
      use_this_oem_value  = 1, # :default
      do_show_the_command = false
    )
    i = i.join(' ').strip if i.is_a? Array
    case use_this_oem_value
    when :default
      use_this_oem_value = 1
    end
    case do_show_the_command
    when :do_show_the_command
      do_show_the_command = true
    end
    use_this_command = "tesseract #{i} stdout -l eng --oem #{use_this_oem_value} --psm 3 2>&1"
    if do_show_the_command
      puts use_this_command
    end
    # ========================================================================= #
    # An alternative to `` may be Open3.capture3(), such as like this:
    #   require 'open3'
    #   text, _,  _ = Open3.capture3(use_this_command)
    #   puts text    
    # ========================================================================= #
    result = `#{use_this_command}`
    return result.strip
  end; self.instance_eval { alias return_text_from_this_image extract_text_from_this_image } # === ImageParadise.return_text_from_this_image

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.extract_text_from_this_image(ARGV)
end # extract_text_from_this_image