#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/toplevel_methods/grayscale_this_image.rb'
# ImageParadise.grayscale_this_image(i, 'NEW_FILE_NAME_GOES_HERE.png')
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/e.rb'

  # ========================================================================= #
  # === ImageParadise.greyscale_this_image
  #
  # This method will greyscale an image file. Right now we depend
  # on the gem mini_magick for this task.
  #
  # This method may be useful when you wish to pre-process a coloured
  # image, turn it into black-white only, before using OCR ("optical
  # character recognition") on it.
  # ========================================================================= #
  def self.greyscale_this_image(
      i,
      use_this_filename = :default_prepended_greyscaled
    )
    begin
      require 'mini_magick'
      i = [i].flatten.compact
      i.each {|this_file|
        this_file = this_file.dup if this_file.frozen?
        img = MiniMagick::Image.open(this_file)
        img.colorspace('Gray') # Turn it into grayscale here.
        # =================================================================== #
        # Next we must determine the correct filename. The method will 
        # assume that if a Symbol is given, that Symbol will have a 
        # special meaning. Based on that, the name will be determined 
        # accordingly.
        # =================================================================== #
        case use_this_filename
        when :default_prepended_grayscaled
          use_this_filename = "grayscaled_#{this_file}"
        when :default_prepended_greyscaled
          use_this_filename = "greyscaled_#{this_file}"
        end
        store_here = use_this_filename
        e "Storing into the file `#{store_here}`."
        img.write(store_here)
      }
    rescue LoadError
      e 'This method requires the gem mini_magick.'
      e
      e 'You can install it via:'
      e
      e '  gem install mini_magick'
      e
    end
  end 

  # ========================================================================= #
  # === ImageParadise.grayscale_this_image
  #
  # This is just for "support" of american english. The resulting image
  # files will have a prepended string called 'grayscaled_' rather
  # than 'greyscaled_'.
  # ========================================================================= #
  def self.grayscale_this_image(
      i, use_this_filename = :default_prepended_grayscaled
    )
    ::ImageParadise.greyscale_this_image(i, use_this_filename)
  end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.greyscale_this_image(ARGV)
end # greyscalethisimage