#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This menu parses the commandline.
# =========================================================================== #
# require 'image_paradise/toplevel_methods/menu.rb'
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/help.rb'
  require 'image_paradise/toplevel_methods/wallpaper.rb'

  # ========================================================================= #
  # === @use_this_image
  # ========================================================================= #
  @use_this_image = nil

  # ========================================================================= #
  # === ImageParadise.use_this_image?
  # ========================================================================= #
  def self.use_this_image?
    @use_this_image
  end

  # ========================================================================= #
  # === ImageParadise.set_use_this_image
  # ========================================================================= #
  def self.set_use_this_image(i)
    @use_this_image = i
  end

  # ========================================================================= #
  # === ImageParadise.menu
  # ========================================================================= #
  def self.menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === imgparadise --gui
      # ===================================================================== #
      when /^-?-?gui$/i,
           /^-?-?gtk$/i,
           /^-?-?gtk(-|_| )?image(-|_| )?manipulation$/i,
           /^-?-?gtk(-|_| )?image$/i
        require 'image_paradise/gui/gtk3/notebook/notebook.rb'
        ::ImageParadise::GUI::Gtk::Notebook.run
      # ===================================================================== #
      # === imgparadise --cfdg-GUI
      # ===================================================================== #
      when /^-?-?cfdg(-|_| )?GUI$/i,
           /^-?-?gtk3$/i # === imgparadise --gtk3
        require 'image_paradise/gui/gtk3/cfdg_widget/cfdg_widget.rb'
        ::ImageParadise::GUI::Gtk::CfdgWidget.run
      # ===================================================================== #
      # === imgparadise FOOBAR.png --rotate=33%
      # ===================================================================== #
      when /-?-?rotate=(-?\d{1,2})%?$/i
        rotate_by_n_percent = $1.to_s.dup
        use_this_file = ''.dup
        if ::ImageParadise.use_this_image?
          use_this_file << ::ImageParadise.use_this_image?.dup
        end
        ::ImageParadise.rotate_this_image(
          use_this_file,
          rotate_by_n_percent,
          :black
        )
      # ===================================================================== #
      # === imgparadise --cfdg
      # ===================================================================== #
      when /^-?-?cfdg$/
        require 'image_paradise/confree_generator/confree_generator.rb'
        ::ImageParadise.confreegenerator('--html')
      # ===================================================================== #
      # === image_paradise --help
      # ===================================================================== #
      when /help/i
        show_help
      # ===================================================================== #
      # === image_paradise --autogenerate-black-wallpaper
      #
      # The variant here will allow us to us any colour for the wallpaper,
      # whereas the variant below will create a default wallpaper.
      # ===================================================================== #
      when /^-?-?autogenerate-?(.+)-?wallpaper/
        use_this_colour = $1.to_s.dup.delete('-')
        ::ImageParadise.wallpaper(colour: use_this_colour)
      # ===================================================================== #
      # === image_paradise --wallpaper
      # ===================================================================== #
      when /-?-?wallpaper?/
        ::ImageParadise.wallpaper
      else
        if File.exist? i
          set_use_this_image(i)
        else
          e 'Not found: '+i.to_s
        end
      end
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.menu(ARGV)
end