#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/toplevel_methods/thumbnail.rb'
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/esystem.rb'

  # ========================================================================= #
  # === ImageParadise.thumbnail
  #
  # This method can be used to generate a thumbnail from a local, existing
  # image.
  #
  # The third argument is the filename for the generated-file. Normally
  # this is inferred automatically, hence why the symbol :infer is
  # used for this, but the user can also provide his or her own filename
  # here.
  # ========================================================================= #
  def self.thumbnail(
      i,
      width_height    = :default,
      output_filename = :infer
    )
    case width_height
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :default_width_and_size
      width_height = '250x90'
    end
    if i.is_a? Array
      i = i.join(' ').strip
    end
    case output_filename
    when :infer
      output_filename = 'thumbnail_'+File.basename(i)
    end
    _ = 'convert '+i+' -thumbnail '+
        width_height.to_s+' '+output_filename
    e
    esystem _
    e
    return output_filename # We also return the filename here.
  end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.thumbnail(ARGV)
end # thumbnail