#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::ToPng
#
# This class will simply convert an image file to .png.
#
# Usage example:
#
#   ImageParadise::ToPng.new
#
# =========================================================================== #
# require 'image_paradise/toplevel_methods/to_png.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'
require 'image_paradise/optimizer.rb'

module ImageParadise

class ToPng < ::ImageParadise::Base # === ImageParadise::ToPng

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = [i].flatten.compact
    @input = i
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?(
      i = @input
    )
    i.gsub(File.extname(i),'')+'.png'
  end

  # ========================================================================= #
  # === run_everything
  #
  # This method contains the main logic of the file.
  # ========================================================================= #
  def run_everything
    @input.each {|this_file|
      store_where = store_where?(this_file)
      opn; e 'Next converting '+simp(this_file)+' to '+sfile(store_where)
      _ = 'convert '+this_file+' '+store_where
      esystem _
      if File.exist? store_where
        opn; e 'We will next try to optimize '+sfile(store_where)
        opn; e 'The old file size is: '+sfancy(File.size(store_where).to_s)
        ::ImageParadise::Optimizer[store_where]
        opn; e 'The new file size is: '+sfancy(File.size(store_where).to_s)
      end
    }
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # (run tag)
    run_everything
  end

end

# =========================================================================== #
# === ImageParadise.to_png
# =========================================================================== #
def self.to_png(i = ARGV)
  ImageParadise::ToPng.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::ToPng.new(ARGV)
end # to_png