#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/toplevel_methods/write_this_text.rb'
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/misc.rb'

  # ========================================================================= #
  # === ImageParadise.write_this_text
  #
  # This will write to an image file.
  #
  # Invocation example:
  #
  #   ImageParadise.write_this_text('Bachelor', {colour: 'red', height: 80, width: 250})
  #
  # ========================================================================= #
  def self.write_this_text(
      text         = DEFAULT_TEXT,
      hash_options = {}
    )
    hash   = hash_options # Shorter pointer.
    height = hash[:height].to_s
    width  = hash[:width].to_s
    # ======================================================================= #
    # use_this_font = ' -font Rekha'
    # use_this_font = " -font 'Nimbus Mono L'"
    # ======================================================================= #
    use_this_font = " -font 'Luxi-Serif-Bold'"
    _ = 'convert '.dup
    # ======================================================================= #
    # === :colour
    # ======================================================================= #
    if hash.has_key? :colour
      _ << " -fill #{hash[:colour]}"
    else
      _ << ' -fill white'
    end
    _ << use_this_font
    _ << ' -background white'
    # ======================================================================= #
    # If the user supplied the value for :pointsize, then we will use it.
    # ======================================================================= #
    if hash.has_key? :pointsize
      _ << ' -pointsize '+hash[:pointsize].to_s
    else # Otherwise we will use a default value.
      _ << ' -pointsize 28'
    end
    # ======================================================================= #
    # Next add the label (aka the corresponding text).
    # ======================================================================= #
    _ << " label:'"+text+"'"
    _ << ' -weight 600'
    _ << ' -gravity center' # Align into the middle.
    # ======================================================================= #
    # width-height setting.
    #
    # See:
    #
    #   https://www.imagemagick.org/script/command-line-options.php#size
    #
    # ======================================================================= #
    _ << " -size #{width}x#{height}"
    _ << " -extent #{width}x#{height}"
    # ======================================================================= #
    # Add a border.
    # ======================================================================= #
    _ << ' -bordercolor black -border 1'
    sanitized_text = text.tr(' ','_').dup
    if hash[:upcase_filename]
      sanitized_text.upcase!
    end
    # ======================================================================= #
    # Next, decide where to store the image file.
    # ======================================================================= #
    store_here = "#{sanitized_text}.png"
    if store_image_files_in_this_directory?
      store_here.prepend(
        store_image_files_in_this_directory?
      )
    end
    _ << " #{store_here}"
    esystem _
  end

end