#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::CreateNewImage
#
# This class will create a new image.
#
# Usage example:
#
#   ImageParadise::CreateNewImage.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise/utility_scripts/create_new_image.rb'
# ImageParadise::CreateNewImage.new(ARGV)
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class CreateNewImage < ::ImageParadise::Base # === ImageParadise::CreateNewImage

  # ========================================================================= #
  # === DELETE_OUTPUT_FILE_IF_IT_EXISTS
  # ========================================================================= #
  DELETE_OUTPUT_FILE_IF_IT_EXISTS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    set_first_commandline_argument(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @use_this_background_colour
    # ======================================================================= #
    @use_this_background_colour = :white
  end

  # ========================================================================= #
  # === set_first_commandline_argument
  # ========================================================================= #
  def set_first_commandline_argument(i)
    i = [i].flatten.compact
    @first_commandline_argument = i
  end

  # ========================================================================= #
  # === this_image_file?
  # ========================================================================= #
  def this_image_file?
    @first_commandline_argument.first.to_s
  end; alias input?      this_image_file? # === input?
       alias first_commandline_argument? this_image_file? # === first_commandline_argument?

  # ========================================================================= #
  # === name_of_the_output_file?
  # ========================================================================= #
  def name_of_the_output_file?
    'OUTPUT_'+first_commandline_argument?.sub(/\.png$/,'').sub(/\.jpg$/,'')+'.png'
  end; alias output_file? name_of_the_output_file? # === output_file?

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments_starting_with_hyphens?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      @use_this_background_colour = i.to_s.delete('--')
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    _ = first_commandline_argument?
    esystem "convert -size #{_} canvas:#{@use_this_background_colour} "+
            name_of_the_output_file?
  end

  # ========================================================================= #
  # === ImageParadise::CreateNewImage[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::CreateNewImage.new(ARGV)
end # createnewimage 640x400
    # createnewimage 640x400 --black