#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::RandomTextToImage
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class RandomTextToImage < ::ImageParadise::Base

  attr_writer :be_verbose

  # ========================================================================= #
  # === DEFAULT_TEXT
  # ========================================================================= #
  DEFAULT_TEXT = 'Hello Ruby!'

  # ========================================================================= #
  # === DEFAULT_FONT
  # ========================================================================= #
  DEFAULT_FONT = 'DejaVu Sans Mono' #  17Bookman-DemiItalic

  # ========================================================================= #
  # === IMAGE_DIRECTORY
  # ========================================================================= #
  IMAGE_DIRECTORY = '/Depot/Images/'

  # ========================================================================= #
  # === IMAGE_TYPE
  # ========================================================================= #
  IMAGE_TYPE = '.png'

  # ========================================================================= #
  # === STORE_INTO_THIS_YAML_FILE
  # ========================================================================= #
  STORE_INTO_THIS_YAML_FILE = 'last_command_for_random_text_to_image.yml'

  # ========================================================================= #
  # === DEFAULT_FONT_SIZE
  # ========================================================================= #
  DEFAULT_FONT_SIZE = '25'

  # ========================================================================= #
  # === BE_VERBOSE
  # ========================================================================= #
  BE_VERBOSE = true

  # ========================================================================= #
  # === RUN_SYSTEM
  # ========================================================================= #
  RUN_SYSTEM = true # Can be change to debug.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i                             = DEFAULT_TEXT,
      append_this_to_saved_filename = '',
      run_already                   = true,
      be_verbose                    = BE_VERBOSE
    )
    set_be_verbose(be_verbose)
    reset
    set_append_this_to_saved_filename(
      append_this_to_saved_filename
    )
    set_text(i)
    set_everything
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset # reset @_ again.
    @_ = 'convert '.dup
  end

  # ========================================================================= #
  # === set_be_verbose
  # ========================================================================= #
  def set_be_verbose(i) # Setter for @be_verbose.
    @be_verbose = i
  end

  # ========================================================================= #
  # === set_text
  # ========================================================================= #
  def set_text(i = DEFAULT_TEXT) # this sets the @text variable.
    _ = STORE_INTO_THIS_YAML_FILE
    if i.is_a? Array
      if i.empty?
        i = DEFAULT_TEXT
      else
        i = i.join(' ').strip
      end
    end
    i = DEFAULT_TEXT if i.nil?
    if i == 'LAST' # treat LAST as special instruction.
      if File.exist? _
        i = YAML.load_file(_)
      else
        opn; e 'Sorry, file '+sfile(_)+' does not exist.'
      end
    end
    i.gsub!("'", "\\\\'") if i.include? "'"
    @text = i
  end

  # ========================================================================= #
  # === run_n_times
  # ========================================================================= #
  def run_n_times(
      n_times = HOW_MANY_TIMES
    ) # use this to create not one but several images.
    n_times.times { |t|
      reset
      set_append_this_to_saved_filename(t+1)
      set_everything
      run
    }
  end

  # ========================================================================= #
  # === set_append_this_to_saved_filename
  # ========================================================================= #
  def set_append_this_to_saved_filename(i)
    @append_this_to_saved_filename = i.to_s
  end

  # ========================================================================= #
  # === set_everything
  # ========================================================================= #
  def set_everything # wrapper to set everything.
    random_plus = rd(225)
    set_width
    set_height
    set_size(
      @width.to_i  * 8+random_plus,
      @height.to_i * 7+random_plus
    )
    set_canvas
    set_font
    set_pointsize
    set_draw1
    set_channel
    set_blur
    set_transparent # if we want a transparent background.
    # set_background_colour
    set_fill
    set_stroke
    set_draw2
    # set_border # Something is weird here.
    set_name # This here should come right before run_sys_cmd is called.
  end

  # ========================================================================= #
  # === set_width
  # ========================================================================= #
  def set_width
    @width = ( 25+random_number(6) ).to_s
  end

  # ========================================================================= #
  # === set_height
  # ========================================================================= #
  def set_height
    @height = ( 25+random_number(6) ).to_s
  end

  # ========================================================================= #
  # === set_draw1
  # ========================================================================= #
  def set_draw1 # this is the draw for text.
    _ = "'"+@text+"'"
    @_ << ' -draw "text '+@width+','+@height+' '+_+'"'
  end

  # ========================================================================= #
  # === set_draw2
  # ========================================================================= #
  def set_draw2 # This is the draw for -stroke
    _ = "'"+@text+"'"
    @_ << ' -draw "text 20,'+DEFAULT_FONT_SIZE+' '+_+'"'
  end

  # ========================================================================= #
  # === set_fill
  # ========================================================================= #
  def set_fill(which_colour = get_random_colour)
    @_ << ' -fill '+which_colour
  end

  # ========================================================================= #
  # === set_canvas
  # ========================================================================= #
  def set_canvas
    @_ << ' canvas:none'
  end

  # ========================================================================= #
  # === set_transparent
  # ========================================================================= #
  def set_transparent
    @_ << ' -transparent white'
  end

  # ========================================================================= #
  # === set_background_colour
  # ========================================================================= #
  def set_background_colour # background tag.
    @_ << ' -background '
    @_ << 'black'
  end

  # ========================================================================= #
  # === random_number
  # ========================================================================= #
  def random_number(i = 20)
    rand(i)
  end; alias rd          random_number # === rd
       alias random_plus random_number # === random_plus

  # ========================================================================= #
  # === set_border
  # ========================================================================= #
  def set_border
    n = random_number.to_s
    @_ << ' -border '+n+'x'+n
    @_ << ' -bordercolor '+random_colour 
  end

  # ========================================================================= #
  # === store_last_command_into_yaml_file
  # ========================================================================= #
  def store_last_command_into_yaml_file
    File.open(STORE_INTO_THIS_YAML_FILE, 'w') {|f| YAML.dump(@text,f)} 
  end

  # ========================================================================= #
  # === set_name
  # ========================================================================= #
  def set_name(
      this_name = 'output'
    )
    this_name = this_name.dup if this_name.frozen?
    this_name << @append_this_to_saved_filename
    this_name << IMAGE_TYPE
    @stored_where = IMAGE_DIRECTORY+'/'+this_name# keep a reference where it is stored.
    @_ << ' '+@stored_where
  end

  # ========================================================================= #
  # === report_where_we_stored_it
  # ========================================================================= #
  def report_where_we_stored_it
    e 'Stored at `'+sfile(@stored_where)+'`.'
  end

  # ========================================================================= #
  # === set_stroke
  # ========================================================================= #
  def set_stroke
    @_ << ' -stroke '+random_colour
  end

  # ========================================================================= #
  # === set_blur
  # ========================================================================= #
  def set_blur
    i = ( 0+random_plus(5) ).to_s 
    @_ << ' -blur 0x'+i
  end

  # ========================================================================= #
  # === set_channel
  # ========================================================================= #
  def set_channel
    @_ << ' -channel RGBA'
  end

  # ========================================================================= #
  # === set_pointsize
  # ========================================================================= #
  def set_pointsize(i = 12+random_plus(75))
    @_ << ' -pointsize '+i.to_s
  end

  # ========================================================================= #
  # === set_font
  # ========================================================================= #
  def set_font(this_font = DEFAULT_FONT)
    this_font = '"'+this_font.to_s+'"' # Pad this first.
    @_ << ' -font '+this_font
  end

  # ========================================================================= #
  # === set_size
  #
  # This method will set the size of the canvas.
  # ========================================================================= #
  def set_size(width = 500, height = 500)
    @_ << "-size #{width}x#{height}" 
  end

  # ========================================================================= #
  # === remove_old_image_file
  # ========================================================================= #
  def remove_old_image_file(i = @stored_where)
    FileUtils.remove(i) if File.exist? i
  end

  # ========================================================================= #
  # === get_random_colour
  #
  # get random colours
  # ========================================================================= #
  def get_random_colour
    Colours.html_colours?.sample
  end; alias random_colour get_random_colour # === random_colour

  # ========================================================================= #
  # === run_sys_cmd
  # ========================================================================= #
  def run_sys_cmd
    remove_old_image_file # remove the image file if it exsts.
    if @be_verbose
      opn; e N+YEL+'Running this convert command next:'
      e '  '+@_
    end
    system @_ if run_system?
  end

  # ========================================================================= #
  # === stored_where?
  # ========================================================================= #
  def stored_where?
    @stored_where
  end; alias stored_where stored_where? # === stored_where

  # ========================================================================= #
  # === run_system?
  # ========================================================================= #
  def run_system?
    RUN_SYSTEM
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # bundle some methods together (run tag).
    run_sys_cmd
    store_last_command_into_yaml_file
    report_where_we_stored_it
  end

end; end

if __FILE__ == $PROGRAM_NAME
  HOW_MANY_TIMES = 15
  HOW_MANY_TIMES.times { |t| ImageParadise::RandomTextToImage.new(ARGV, t+1) }
end # rtext "We are the BORG."