#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::ToGif
#
# This class will simply convert an image file to .gif.
#
# Usage example:
#
#   ImageParadise::ToGif.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise/utility_scripts/to_gif.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class ToGif < ::ImageParadise::Base # === ImageParadise::ToGif

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === run_everything
  #
  # This method contains the main logic of the file.
  # ========================================================================= #
  def run_everything
    opn; e 'Next converting '+simp(input?)+' to '+sfile(store_where?)
    _ = 'convert '+input?+' '+store_where?
    esystem _
    if File.exist? store_where?
      opn; e 'We will next try to optimize '+sfile(store_where?)
      opn; e 'The old file size is: '+sfancy(File.size(store_where?).to_s)
      ImageParadise::Optimizer[store_where?]
      opn; e 'The new file size is: '+sfancy(File.size(store_where?).to_s)
    end
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    input?.gsub(File.extname(input?),'')+'.gif'
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # (run tag)
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::ToGif.new(ARGV)
end # to_gif